#include <ESP8266WiFi.h>

#ifdef ESP8266
  extern "C" {
    #include "user_interface.h"
  }
  #include "spiESP.h"
  #include "spiEspDirect.h"
  #include "loraESP.h"
#endif

#define WITHOUT_CRC 0x00
#define WITH_CRC 0x01
#define ADD_CRC 0x02
#define MODE_RX 0x80
#define N_MAX 64
#define TIMEOUT_ms       100
unsigned char ucSt[N_MAX];


SpiEsp spiESP;
SpiEspDirect spiEspDirect(&spiESP);
LoRaESP sx1276(&spiEspDirect);

void vAnswer(byte* ucSt,int len){
  sx1276.mode_tx(ucSt,len);
}

void vAnswerCheckingCRC(byte* ucSt,int len){
  int i;
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len-2);

  if((byte)(uiCRC >> 8) == ucSt[len-1] && (byte)(uiCRC & 0xFF) == ucSt[len-2]){ 
    sx1276.mode_tx(ucSt,len);
  }else{
    Serial.println("   CRC doesn't match   ");
  }
}

void vAnswerAddingCRC(byte* ucSt,int len){
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len);
  
  ucSt[len] = (byte)(uiCRC & 0xFF);
  ucSt[len+1] = (byte)(uiCRC >> 8);
  
  sx1276.mode_tx(ucSt,len+2);
}

void vProcessa(byte* ucSt,int len){
  if(ucSt[0] & MODE_RX){            // 1st byte is function mode
    Serial.println("  Rx mode");
    sx1276.bRxMode = true;
    sx1276.bTxMode = false;
  }else{
    if(ucSt[1] == len-2){           // 2nd byte is payload length
      sx1276.bRxMode = false;
      sx1276.bTxMode = true;
      switch(ucSt[0]){
        case WITHOUT_CRC:
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC:
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC:
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        default:
             vAnswer(ucSt,len);
     }
    }else
      Serial.println("Length doesn't match");
  }
}

void setup() {
  Serial.begin(9600);
  sx1276.init_SX1276();
}

void loop() {
  int i,nCmpt = 0;

  //digitalWrite(LED1,LOW); 
  //digitalWrite(LED2,LOW);  
  sx1276.reset_sx1276(); 
  sx1276.Config_SX1276();
  
  while(1){
    while(Serial.available() > 0) {
      ucSt[nCmpt] = (unsigned char)Serial.read();
      nCmpt++;
      delay(2);
    }
    if(nCmpt){
      vProcessa(ucSt,nCmpt);
    }else{
    if(sx1276.bRxMode){
      if(sx1276.bReadLora(ucSt,N_MAX)){
        for(i = 0; i < ucSt[1] ; i++) 
        Serial.write(ucSt[i]);
      }
    }
    }
    nCmpt = 0;
  }     
}

